function [xopt topt] = half_exhaustive(r,n_plus,n_minus,L)
% function for solving the optimization problem via exhaustive search,
% see problem (33) in F. Altenbach, G. Böcherer, R. Mathar, Short Huffman
% Codes Producing 1s Half of the Time. half_exhaustive is called by halfhc.
%
% INPUT
%   r       : pmf of codeword lengths
%   n_plus  : vector of maximum expected number of 1s
%   n_minus : vector of minimum expected number of 1s
%   L       : average length
%
% OUTPUT
%   xopt    : optimal selection vector
%   topt    : optimal value, corresponds to |q-0.5|
%
%   See also half_bisection, halfhc, huffmandict
%
% tested for Matlab R2010b
% 22 July 2011
% (c) Georg Böcherer, Fabian Altenbach
% email: geborg@gmail.com, fabian.altenbach@googlemail.com

m = length(r);

solutions = combn([true false],m);
num_sol = length(solutions(:,1));

topt = abs(n_plus'*r/L-0.5);
xopt = zeros(m,1);

for k=1:num_sol
    x = solutions(k,:)';
    t_tmp = abs((n_plus+x.*(n_minus-n_plus))'*r/L-0.5);
    if(t_tmp<topt)
        topt = t_tmp;
        xopt = x;
    end
end
